--USE CREDIT
go

-- First create these single column indexes
CREATE INDEX MemberLastName ON Member(LastName)
CREATE INDEX MemberFirstName ON Member(FirstName)
go

-- Review your index list
EXEC sp_helpindex member
go
-- Or use the system tables...
SELECT * FROM sysindexes WHERE id = object_id('member') 
go

-- Check space usage
EXEC sp_spaceused member, TRUE
go

SET STATISTICS IO ON
go

-- WITH ONLY the indexes created as above this query does
-- as best it can by joining two indexes to cover the query!
-- Great but not as optimal as it could be?!

SELECT Member_No, FirstName, Region_No
FROM Member 
WHERE FirstName LIKE 'K%'   -- guess 1/26 of table
        AND Region_No > 6 -- guess 1/3 of table
        AND Member_No < 5000  -- 1/2 of table
--index fname, member_no 1/26 * 1/2 = 1/52
--index region_no, member_no 1/3 * 1/2 = 1/6

-- What if we could only use one index? Would it be worthwhile?
-- NO BUT prior to 7.0 SQL Server could only use one index per table
-- per query...these two queries are worse than a table scan!
SELECT Member_No, FirstName, Region_No
FROM Member (index = MemberFirstName)
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

SELECT Member_No, FirstName, Region_No
FROM Member (index = member_region_link)
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

-- SO - using two indexes is certainly a better choice!!! If you ever 
-- want to force multiple indexes, the index order IS IMPORTANT.
-- Put the more selective index first!
-- Compare these two and you'll see the first is faster!
SELECT Member_No, FirstName, Region_No
FROM Member (index (MemberFirstName, member_region_link))
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

SELECT Member_No, FirstName, Region_No
FROM Member (index (member_region_link, MemberFirstName))
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

-- BUT the best choice for indexing for AND is...
-- (1) Create an index on ANY very selective column. If something 
-- is very selective then that's the only index you need!
-- (2) Create in index on ANY combination of selective criteria. You
-- don't need everything from the WHERE clause but try and supply
-- the combination of columns that will yield a selective set!
-- (3) If nothing is selective - even when combined - then cover
-- the query!

CREATE INDEX MemberCovering ON member(firstname, region_no, member_no)
-- IS identical to...
-- CREATE INDEX MemberCovering ON member(firstname, region_no) -- member_no is CL
-- because the member_no column is the clustered index 
-- and therefore already included in the index.

SELECT Member_No, FirstName, Region_No
FROM Member
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000
go

-- What if our index doesn't cover!
-- AND it's very low selectivity THEN the best index might be 
-- NOT to use one...TABLE SCAN
SELECT Member_No, FirstName, Region_No, phone_no
FROM Member 
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

SELECT Member_No, FirstName, Region_No, phone_no
FROM Member (index = MemberFirstName)
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

SELECT Member_No, FirstName, Region_No, phone_no
FROM Member (index = member_region_link)
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

-- When forcing multiple indexes - order may be relevant
SELECT Member_No, FirstName, Region_No, phone_no
FROM Member (index (MemberFirstName, member_region_link))
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

SELECT Member_No, FirstName, Region_No, phone_no
FROM Member (index (member_region_link, MemberFirstName))
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

SELECT Member_No, FirstName, Region_No, phone_no
FROM Member (index (MemberCovering))
WHERE FirstName LIKE 'K%' 
        AND Region_No > 6 AND Member_No < 5000

-- DROP INDEX Member.MemberLastName
-- DROP INDEX Member.MemberFirstName
-- DROP INDEX Member.MemberCovering